@@ -91,6 +91,7 @@ ei_device_destroy(struct ei_device *device)
 	ei_keyboard_unref(device->keyboard);
 	ei_seat_unref(seat);
 	free(device->name);
+	free(device->pending_region_mapping_id);
 }
 
 _public_
@@ -194,11 +195,26 @@ handle_msg_region(struct ei_device *device, uint32_t x, uint32_t y,
 	ei_region_set_offset(r, x, y);
 	ei_region_set_size(r, w, h);
 	ei_region_set_physical_scale(r, scale);
+
+	_cleanup_free_ char *mapping_id = steal(&device->pending_region_mapping_id);
+	if (mapping_id)
+		ei_region_set_mapping_id(r, mapping_id);
+
 	ei_device_add_region(device, r);
 
 	return NULL;
 }
 
+static struct brei_result *
+handle_msg_region_mapping_id(struct ei_device *device, const char *mapping_id)
+{
+	if (device->pending_region_mapping_id)
+		return brei_result_new(EI_CONNECTION_DISCONNECT_REASON_PROTOCOL,
+				       "EIS sent the region mapping_id twice");
+	device->pending_region_mapping_id = xstrdup(mapping_id);
+	return NULL;
+}
+
 static struct brei_result *
 handle_msg_done(struct ei_device *device)
 {
@@ -464,6 +480,8 @@ static const struct ei_device_interface interface = {
 	.stop_emulating = handle_msg_stop_emulating,
 	.frame = handle_msg_frame,
 	.interface = handle_msg_interface,
+	/* v2 */
+	.region_mapping_id = handle_msg_region_mapping_id,
 };
 
 const struct ei_device_interface *
@@ -1195,6 +1213,18 @@ ei_device_get_region(struct ei_device *device, size_t index)
 	return list_nth_entry(struct ei_region, &device->regions, link, index);
 }
 
+_public_ struct ei_region *
+ei_device_get_region_at(struct ei_device *device, double x, double y)
+{
+	struct ei_region *r;
+
+	list_for_each(r, &device->regions, link) {
+		if (ei_region_contains(r, x, y))
+			return r;
+	}
+	return NULL;
+}
+
 static inline void
 ei_device_resume_scrolling(struct ei_device *device, double x, double y)
 {
