@@ -193,6 +193,18 @@ eis_device_get_region(struct eis_device *device, size_t index)
 	return list_nth_entry(struct eis_region, &device->regions, link, index);
 }
 
+_public_ struct eis_region *
+eis_device_get_region_at(struct eis_device *device, double x, double y)
+{
+	struct eis_region *r;
+
+	list_for_each(r, &device->regions, link) {
+		if (eis_region_contains(r, x, y))
+			return r;
+	}
+	return NULL;
+}
+
 _public_ struct eis_client *
 eis_device_get_client(struct eis_device *device)
 {
@@ -807,6 +819,17 @@ eis_device_add(struct eis_device *device)
 	if (device->type == EIS_DEVICE_TYPE_VIRTUAL) {
 		struct eis_region *r;
 		list_for_each(r, &device->regions, link) {
+			if (r->mapping_id) {
+				if (client->interface_versions.ei_device >= EIS_DEVICE_EVENT_REGION_MAPPING_ID_SINCE_VERSION) {
+					rc = eis_device_event_region_mapping_id(device, r->mapping_id);
+					if (rc < 0)
+						goto out;
+				} else {
+					/* If our client doesn't support mapping_id, drop it */
+					free(r->mapping_id);
+					r->mapping_id = NULL;
+				}
+			}
 			rc = eis_device_event_region(device, r->x, r->y, r->width, r->height, r->physical_scale);
 			if (rc < 0)
 				goto out;
