@@ -123,7 +123,7 @@ xsend_with_fd(int fd, const void *buf, size_t len, int *fds)
 struct iobuf {
 	size_t sz;
 	size_t len;
-	char *data;
+	uint8_t *data;
 	int fds[32];
 };
 
@@ -131,7 +131,7 @@ struct iobuf *
 iobuf_new(size_t size)
 {
 	struct iobuf *buf = malloc(sizeof(*buf));
-	char *data = malloc(size);
+	uint8_t *data = malloc(size);
 
 	assert(buf);
 	assert(data);
@@ -182,7 +182,7 @@ iobuf_pop(struct iobuf *buf, size_t nbytes)
  * The returned pointer only valid in the immediate scope, any iobuf
  * function may invalidate the pointer.
  */
-const char *
+const uint8_t *
 iobuf_data(struct iobuf *buf)
 {
 	return buf->data;
@@ -194,7 +194,7 @@ iobuf_data(struct iobuf *buf)
  * The returned pointer only valid in the immediate scope, any iobuf
  * function may invalidate the pointer.
  */
-const char *
+const uint8_t *
 iobuf_data_end(struct iobuf *buf)
 {
 	return buf->data + buf->len;
@@ -209,28 +209,36 @@ iobuf_take_fd(struct iobuf *buf)
 {
 	int fd = buf->fds[0];
 	if (fd != -1)
-		memmove(buf->fds, buf->fds + 1, ARRAY_LENGTH(buf->fds) - 1);
+		memmove(buf->fds, buf->fds + 1, (ARRAY_LENGTH(buf->fds) - 1) * sizeof(*buf->fds));
 	return fd;
 }
 
 static inline void
 iobuf_resize(struct iobuf *buf, size_t to_size)
 {
-	char *newdata = realloc(buf->data, to_size);
+	uint8_t *newdata = realloc(buf->data, to_size);
 	assert(newdata);
 
 	buf->data = newdata;
 	buf->sz = to_size;
 }
 
+static inline void
+iobuf_extend(struct iobuf *buf, size_t extra)
+{
+	size_t newsize = buf->len + extra;
+	if (newsize > buf->sz)
+		iobuf_resize(buf, newsize);
+}
+
 /**
  * Remove the data bytes from the buffer. The caller must free() the data.
  * The buffer state is the same as iobuf_new() after this call.
  */
-char *
+uint8_t *
 iobuf_take_data(struct iobuf *buf)
 {
-	char *data = buf->data;
+	uint8_t *data = buf->data;
 
 	buf->data = NULL;
 	buf->len = 0;
@@ -244,25 +252,51 @@ iobuf_take_data(struct iobuf *buf)
  * size it is resized automatically.
  */
 void
-iobuf_append(struct iobuf *buf, const char *data, size_t len)
+iobuf_append(struct iobuf *buf, const void *data, size_t len)
 {
 	if (len == 0)
 		return;
 
-	if (buf->len + len > buf->sz) {
-		size_t newsize = buf->len + len;
-		iobuf_resize(buf, newsize);
-	}
-
+	iobuf_extend(buf, len);
 	memcpy(buf->data + buf->len, data, len);
 	buf->len += len;
 }
 
+void
+iobuf_append_u32(struct iobuf *buf, uint32_t data)
+{
+	size_t len = 4;
+
+	iobuf_extend(buf, len);
+	memcpy(buf->data + buf->len, &data, len);
+	buf->len += len;
+}
+
+void
+iobuf_append_u64(struct iobuf *buf, uint64_t data)
+{
+	size_t len = 8;
+
+	iobuf_extend(buf, len);
+	memcpy(buf->data + buf->len, &data, len);
+	buf->len += len;
+}
+
+void
+iobuf_append_f32(struct iobuf *buf, float data)
+{
+	size_t len = 4;
+
+	iobuf_extend(buf, len);
+	memcpy(buf->data + buf->len, &data, len);
+	buf->len += len;
+}
+
 /**
  * Prepend the given data to the buffer.
  */
 void
-iobuf_prepend(struct iobuf *buf, const char *data, size_t len)
+iobuf_prepend(struct iobuf *buf, const void *data, size_t len)
 {
 	if (len == 0)
 		return;
@@ -281,7 +315,7 @@ int
 iobuf_append_fd(struct iobuf *buf, int fd)
 {
 	/* Array must remain terminated by -1 */
-	for (size_t idx = 0; idx < sizeof(buf->fds) - 1; idx ++) {
+	for (size_t idx = 0; idx < ARRAY_LENGTH(buf->fds) - 1; idx ++) {
 		if (buf->fds[idx] == -1) {
 			int f = dup(fd);
 			if (f == -1)
@@ -427,6 +461,28 @@ MUNIT_TEST(test_iobuf_cleanup)
 	return MUNIT_OK;
 }
 
+MUNIT_TEST(test_iobuf_take_fd)
+{
+	_cleanup_iobuf_ struct iobuf *buf = iobuf_new(10);
+	const size_t nfds = ARRAY_LENGTH(buf->fds);
+	int *last_fd = &buf->fds[nfds - 1]; /* always -1 */
+
+	for (size_t i = 0; i < nfds - 1; i++) {
+		buf->fds[i] = 10 + i;
+	}
+
+	for (size_t i = 0; i < nfds - 1; i++) {
+		int fd = iobuf_take_fd(buf);
+		munit_assert_int(fd, ==, 10 + i);
+		munit_assert_int(*last_fd, ==, -1);
+	}
+
+	int fd = iobuf_take_fd(buf);
+	munit_assert_int(fd, ==, -1);
+
+	return MUNIT_OK;
+}
+
 MUNIT_TEST(test_iobuf_append_prepend)
 {
 	/* Test appending data */
@@ -442,7 +498,7 @@ MUNIT_TEST(test_iobuf_append_prepend)
 	munit_assert_size(buf->sz, ==, 10);
 
 	/* we don't have a trailing \0 */
-	const char *bufdata = iobuf_data(buf);
+	const uint8_t *bufdata = iobuf_data(buf);
 	munit_assert_char(bufdata[0], ==, 'f');
 	munit_assert_char(bufdata[1], ==, 'o');
 	munit_assert_char(bufdata[2], ==, 'o');
@@ -474,7 +530,7 @@ MUNIT_TEST(test_iobuf_append_prepend)
 	munit_assert_size(iobuf_len(buf), ==, expected_size);
 	munit_assert_size(buf->sz, ==, expected_size);
 	/* now we have a trailing \0 */
-	munit_assert_string_equal(iobuf_data(buf), "barfoodata forcing resize");
+	munit_assert_string_equal((const char *)iobuf_data(buf), "barfoodata forcing resize");
 
 	/* and again with prepending */
 	const char prepend_data2[] = "second resize";
@@ -483,7 +539,48 @@ MUNIT_TEST(test_iobuf_append_prepend)
 
 	munit_assert_size(iobuf_len(buf), ==, expected_size);
 	munit_assert_size(buf->sz, ==, expected_size);
-	munit_assert_string_equal(iobuf_data(buf), "second resizebarfoodata forcing resize");
+	munit_assert_string_equal((const char *)iobuf_data(buf), "second resizebarfoodata forcing resize");
+
+	return MUNIT_OK;
+}
+
+MUNIT_TEST(test_iobuf_append_values)
+{
+	_cleanup_iobuf_ struct iobuf *buf = iobuf_new(10);
+
+	iobuf_append_u32(buf, -1);
+	size_t expected_size = 4;
+
+	munit_assert_size(buf->len, ==, expected_size);
+	munit_assert_size(iobuf_len(buf), ==, expected_size);
+	munit_assert_size(buf->sz, ==, 10);
+
+	const uint8_t *bufdata = iobuf_data(buf);
+	munit_assert_int(bufdata[0], ==, 0xff);
+	munit_assert_int(bufdata[1], ==, 0xff);
+	munit_assert_int(bufdata[2], ==, 0xff);
+	munit_assert_int(bufdata[3], ==, 0xff);
+
+	free(iobuf_take_data(buf)); /* drops and replaces current buffer */
+
+	iobuf_append_u64(buf, 0xabababababababab);
+	expected_size = 8;
+
+	munit_assert_size(buf->len, ==, expected_size);
+	munit_assert_size(iobuf_len(buf), ==, expected_size);
+	munit_assert_size(buf->sz, ==, 10);
+
+	bufdata = iobuf_data(buf);
+	munit_assert_int((unsigned char)bufdata[0], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[1], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[2], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[3], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[4], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[5], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[6], ==, 0xab);
+	munit_assert_int((unsigned char)bufdata[7], ==, 0xab);
+
+	free(iobuf_take_data(buf));
 
 	return MUNIT_OK;
 }
@@ -502,7 +599,7 @@ MUNIT_TEST(test_iobuf_prepend_empty_buffer)
 	munit_assert_size(buf->sz, ==, 10);
 
 	/* we don't have a trailing \0 */
-	const char *bufdata = iobuf_data(buf);
+	const uint8_t *bufdata = iobuf_data(buf);
 	munit_assert_char(bufdata[0], ==, 'f');
 	munit_assert_char(bufdata[1], ==, 'o');
 	munit_assert_char(bufdata[2], ==, 'o');
@@ -521,7 +618,7 @@ MUNIT_TEST(test_iobuf_pop)
 	munit_assert_size(iobuf_len(buf), ==, 3);
 
 	/* we don't have a trailing \0 */
-	const char *bufdata = iobuf_data(buf);
+	const uint8_t *bufdata = iobuf_data(buf);
 	munit_assert_char(bufdata[0], ==, 'b');
 	munit_assert_char(bufdata[1], ==, 'a');
 	munit_assert_char(bufdata[2], ==, 'r');
@@ -543,8 +640,7 @@ MUNIT_TEST(test_iobuf_append_short)
 	munit_assert_size(buf->len, ==, 4);
 	munit_assert_size(iobuf_len(buf), ==, 4);
 	munit_assert_size(buf->sz, ==, 10);
-	munit_assert_string_equal(iobuf_data(buf),
-				  "foo");
+	munit_assert_string_equal((const char *)iobuf_data(buf), "foo");
 
 	return MUNIT_OK;
 }
@@ -574,7 +670,7 @@ MUNIT_TEST(test_iobuf_append_fd)
 	/* so we can do strcmp */
 	const char nullbyte = '\0';
 	iobuf_append(buf, &nullbyte, 1);
-	munit_assert_string_equal(iobuf_data(buf), "foob");
+	munit_assert_string_equal((const char *)iobuf_data(buf), "foob");
 
 	/* read when there's nothing waiting */
 	int blocking_read = iobuf_append_from_fd(buf, rd);
@@ -607,6 +703,29 @@ MUNIT_TEST(test_iobuf_append_fd)
 	return MUNIT_OK;
 }
 
+MUNIT_TEST(test_iobuf_append_fd_too_many)
+{
+	_cleanup_fclose_ FILE *fp = tmpfile();
+	int fd = fileno(fp);
+
+	_cleanup_iobuf_ struct iobuf *buf = iobuf_new(20);
+	const size_t nfds = ARRAY_LENGTH(buf->fds);
+	int *last_fd = &buf->fds[nfds - 1]; /* always -1 */
+	int err = 0;
+	size_t count = 0;
+
+	/* 32 fds hardcoded in the struct, last one is always -1 */
+	for (count = 0; err == 0 && count < nfds + 1; count++) {
+		err = iobuf_append_fd(buf, fd);
+		munit_assert_int(*last_fd, ==, -1);
+	}
+
+	munit_assert_int(count, ==, 32);
+	munit_assert_int(err, ==, -ENOMEM);
+
+	return MUNIT_OK;
+}
+
 MUNIT_TEST(test_iobuf_recv_fd)
 {
 	int fds[2];
