@@ -516,6 +516,7 @@ MUNIT_TEST(test_ei_device_regions)
 		_unref_(eis_region) *r1 = eis_device_new_region(device);
 		eis_region_set_size(r1, 100, 200);
 		eis_region_set_offset(r1, 300, 400);
+		eis_region_set_mapping_id(r1, "oo oo eye dee");
 		/* no scale, default to 1.0 */
 		eis_region_add(r1);
 
@@ -543,7 +544,7 @@ MUNIT_TEST(test_ei_device_regions)
 
 	with_client(peck) {
 		struct ei_device *device = peck_ei_get_default_pointer_absolute(peck);
-		struct ei_region *r;
+		struct ei_region *r, *r2;
 
 		r = ei_device_get_region(device, 0);
 		munit_assert_int(ei_region_get_width(r), ==, 100);
@@ -551,6 +552,12 @@ MUNIT_TEST(test_ei_device_regions)
 		munit_assert_int(ei_region_get_x(r), ==, 300);
 		munit_assert_int(ei_region_get_y(r), ==, 400);
 		munit_assert_double_equal(ei_region_get_physical_scale(r), 1.0, 2 /* precision */);
+		munit_assert_string_equal(ei_region_get_mapping_id(r), "oo oo eye dee");
+
+		r2 = ei_device_get_region_at(device, 300, 400);
+		munit_assert_ptr_equal(r, r2);
+		r2 = ei_device_get_region_at(device, 350, 450);
+		munit_assert_ptr_equal(r, r2);
 
 		r = ei_device_get_region(device, 1);
 		munit_assert_int(ei_region_get_width(r), ==, 500);
@@ -558,6 +565,10 @@ MUNIT_TEST(test_ei_device_regions)
 		munit_assert_int(ei_region_get_x(r), ==, 700);
 		munit_assert_int(ei_region_get_y(r), ==, 800);
 		munit_assert_double_equal(ei_region_get_physical_scale(r), 3.9, 2 /* precision */);
+		munit_assert_null(ei_region_get_mapping_id(r));
+
+		r2 = ei_device_get_region_at(device, 750, 850);
+		munit_assert_ptr_equal(r, r2);
 
 		r = ei_device_get_region(device, 2);
 		munit_assert_int(ei_region_get_width(r), ==, 900);
@@ -565,8 +576,14 @@ MUNIT_TEST(test_ei_device_regions)
 		munit_assert_int(ei_region_get_x(r), ==, 1100);
 		munit_assert_int(ei_region_get_y(r), ==, 1200);
 		munit_assert_double_equal(ei_region_get_physical_scale(r), 0.3, 2 /* precision */);
+		munit_assert_null(ei_region_get_mapping_id(r));
+
+		r2 = ei_device_get_region_at(device, 1999, 2199);
+		munit_assert_ptr_equal(r, r2);
+		munit_assert_ptr_null(ei_device_get_region_at(device, 2000, 2200)); /* at the edge */
 
 		munit_assert_ptr_null(ei_device_get_region(device, 3));
+		munit_assert_ptr_null(ei_device_get_region_at(device, 0, 100));
 	}
 
 	return MUNIT_OK;
