@@ -260,6 +260,7 @@ add_device(struct eis_demo_server *server, struct eis_client *client,
 		eis_device_configure_capability(ptr, EIS_DEVICE_CAP_BUTTON);
 		eis_device_configure_capability(ptr, EIS_DEVICE_CAP_SCROLL);
 		_unref_(eis_region) *rel_region = eis_device_new_region(ptr);
+		eis_region_set_mapping_id(rel_region, "demo region");
 		eis_region_set_size(rel_region, 1920, 1080);
 		eis_region_set_offset(rel_region, 0, 0);
 		eis_region_add(rel_region);
@@ -280,6 +281,7 @@ add_device(struct eis_demo_server *server, struct eis_client *client,
 		eis_device_configure_capability(abs, EIS_DEVICE_CAP_BUTTON);
 		eis_device_configure_capability(abs, EIS_DEVICE_CAP_SCROLL);
 		_unref_(eis_region) *region = eis_device_new_region(abs);
+		eis_region_set_mapping_id(region, "demo region");
 		eis_region_set_size(region, 1920, 1080);
 		eis_region_set_offset(region, 0, 0);
 		eis_region_add(region);
